/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef JNPP_COMMON
#define JNPP_COMMON

#define JNPP_STATUS_INTERNAL_ERROR       0x10000001

#include <stdlib.h>
#include <jni.h>
#include <nppi.h>
#include <npps.h>

#include "Logger.hpp"
#include "JNIUtils.hpp"
#include "PointerUtils.hpp"


extern jfieldID cuComplex_x; // float
extern jfieldID cuComplex_y; // float

extern jfieldID cuDoubleComplex_x; // double
extern jfieldID cuDoubleComplex_y; // double

extern jfieldID NppiPoint_x; // int
extern jfieldID NppiPoint_y; // int

extern jfieldID NppiSize_width; // int
extern jfieldID NppiSize_height; // int

extern jfieldID NppiRect_x; // int
extern jfieldID NppiRect_y; // int
extern jfieldID NppiRect_width; // int
extern jfieldID NppiRect_height; // int


extern jfieldID Npp16sc_re; // short
extern jfieldID Npp16sc_im; // short

extern jfieldID Npp32sc_re; // int
extern jfieldID Npp32sc_im; // int

extern jfieldID Npp64sc_re; // long
extern jfieldID Npp64sc_im; // long

extern jfieldID Npp32fc_re; // float
extern jfieldID Npp32fc_im; // float

extern jfieldID Npp64fc_re; // double
extern jfieldID Npp64fc_im; // double


// Class and method ID for TypedPointer and its constructor
extern jclass TypedPointer_Class;
extern jmethodID TypedPointer_Constructor;


jobject createPointer(JNIEnv *env, jlong pointerValue);
bool initNative(JNIEnv *env, jobject input, NppiPoint &output);
bool initNative(JNIEnv *env, jobject input, NppiSize &output);
bool initNative(JNIEnv *env, jobject input, NppiRect &output);
bool initNative(JNIEnv *env, jobject input, Npp16sc &output);
bool initNative(JNIEnv *env, jobject input, Npp32sc &output);
bool initNative(JNIEnv *env, jobject input, Npp64sc &output);
bool initNative(JNIEnv *env, jobject input, Npp32fc &output);
bool initNative(JNIEnv *env, jobject input, Npp64fc &output);
bool releaseNative(JNIEnv *env, Npp16sc &output, jobject input);
bool releaseNative(JNIEnv *env, Npp32sc &output, jobject input);
bool releaseNative(JNIEnv *env, Npp64sc &output, jobject input);
bool releaseNative(JNIEnv *env, Npp32fc &output, jobject input);
bool releaseNative(JNIEnv *env, Npp64fc &output, jobject input);
bool initNative(JNIEnv *env, jobjectArray input, double output[4][2]);
bool releaseNative(JNIEnv *env, double output[4][2], jobjectArray input);
bool initNative(JNIEnv *env, jobjectArray input, double output[2][3]);
bool releaseNative(JNIEnv *env, double output[2][3], jobjectArray input);
bool initNative(JNIEnv *env, jobjectArray input, Npp32f output[3][4]);
bool releaseNative(JNIEnv *env, Npp32f output[3][4], jobjectArray input);
bool initNative(JNIEnv *env, jbyteArray input, Npp8s output[], int size);
bool releaseNative(JNIEnv *env, Npp8s output[], jbyteArray input, int size);
bool initNative(JNIEnv *env, jbyteArray input, Npp8u output[], int size);
bool releaseNative(JNIEnv *env, Npp8u output[], jbyteArray input, int size);
bool initNative(JNIEnv *env, jshortArray input, Npp16s output[], int size);
bool releaseNative(JNIEnv *env, Npp16s output[], jshortArray input, int size);
bool initNative(JNIEnv *env, jshortArray input, Npp16u output[], int size);
bool releaseNative(JNIEnv *env, Npp16u output[], jshortArray input, int size);
bool initNative(JNIEnv *env, jintArray input, Npp32s output[], int size);
bool releaseNative(JNIEnv *env, Npp32s output[], jintArray input, int size);
bool initNative(JNIEnv *env, jintArray input, Npp32u output[], int size);
bool releaseNative(JNIEnv *env, Npp32u output[], jintArray input, int size);
bool initNative(JNIEnv *env, jlongArray input, Npp64s output[], int size);
bool releaseNative(JNIEnv *env, Npp64s output[], jlongArray input, int size);
bool initNative(JNIEnv *env, jlongArray input, Npp64u output[], int size);
bool releaseNative(JNIEnv *env, Npp64u output[], jlongArray input, int size);
bool initNative(JNIEnv *env, jfloatArray input, Npp32f output[], int size);
bool releaseNative(JNIEnv *env, Npp32f output[], jfloatArray input, int size);
bool initNative(JNIEnv *env, jdoubleArray input, Npp64f output[], int size);
bool releaseNative(JNIEnv *env, Npp64f output[], jdoubleArray input, int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp16sc output[], int size);
bool releaseNative(JNIEnv *env, Npp16sc output[], jobjectArray input, int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp32sc output[], int size);
bool releaseNative(JNIEnv *env, Npp32sc output[], jobjectArray input, int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp32fc output[], int size);
bool releaseNative(JNIEnv *env, Npp32fc output[], jobjectArray input, int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp64fc output[], int size);
bool releaseNative(JNIEnv *env, Npp64fc output[], jobjectArray input, int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp8u* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp8u* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp16s* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp16s* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp16u* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp16u* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp32s* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp32s* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp32f* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp32f* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, Npp64f* output[], int size);
bool initNative(JNIEnv *env, jobjectArray input, const Npp64f* output[], int size);

#endif