/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"
#include "JNpps.hpp"


//=== Auto-generated part: ===================================================

/**
 * <pre>
 * 8-bit unsigned signal allocator.
 * @param nSize Number of unsigned chars in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_18uNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_8u(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp8u* jniResult_native = nppsMalloc_8u(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_18sNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_8s(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp8s* jniResult_native = nppsMalloc_8s(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit unsigned signal allocator.
 * @param nSize Number of unsigned shorts in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_116uNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_16u(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp16u* jniResult_native = nppsMalloc_16u(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit signal allocator.
 * @param nSize Number of shorts in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_116sNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_16s(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp16s* jniResult_native = nppsMalloc_16s(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_116scNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_16sc(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp16sc* jniResult_native = nppsMalloc_16sc(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit unsigned signal allocator.
 * @param nSize Number of unsigned ints in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_132uNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_32u(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp32u* jniResult_native = nppsMalloc_32u(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit integer signal allocator.
 * @param nSize Number of ints in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_132sNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_32s(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp32s* jniResult_native = nppsMalloc_32s(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_132scNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_32sc(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp32sc* jniResult_native = nppsMalloc_32sc(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit float signal allocator.
 * @param nSize Number of floats in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_132fNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_32f(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp32f* jniResult_native = nppsMalloc_32f(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit complex float signal allocator.
 * @param nSize Number of complex float values in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_132fcNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_32fc(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp32fc* jniResult_native = nppsMalloc_32fc(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 64-bit long integer signal allocator.
 * @param nSize Number of long ints in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_164sNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_64s(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp64s* jniResult_native = nppsMalloc_64s(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 64-bit complex long integer signal allocator.
 * @param nSize Number of complex long int values in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_164scNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_64sc(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp64sc* jniResult_native = nppsMalloc_64sc(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 64-bit float (double) signal allocator.
 * @param nSize Number of doubles in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_164fNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_64f(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp64f* jniResult_native = nppsMalloc_64f(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 64-bit complex complex signal allocator.
 * @param nSize Number of complex double valuess in the new signal.
 * @return
 *  A pointer to the new signal. 0 (NULL-pointer) indicates
 *         that an error occurred during allocation.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNpps_nppsMalloc_164fcNative(JNIEnv *env, jclass cls, jint nSize)
{
    // Null-checks for non-primitive arguments
    // nSize is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMalloc_64fc(nSize=%d)\n",
        nSize);

    // Native variable declarations
    int nSize_native = 0;

    // Obtain native variable values
    nSize_native = (int)nSize;

    // Native function call
    Npp64fc* jniResult_native = nppsMalloc_64fc(nSize_native);

    // Write back native variable values
    // nSize is primitive

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * Free method for any 2D allocated memory.
 * This method should be used to free memory allocated with any of the
 * nppiMalloc_<modifier> methods.
 * @param pValues A pointer to memory allocated using nppiMalloc_<modifier>.
 * </pre>
 */
JNIEXPORT void JNICALL Java_jcuda_jnpp_JNpps_nppsFreeNative(JNIEnv *env, jclass cls, jobject pValues)
{
    // Null-checks for non-primitive arguments
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppsFree");
        return;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsFree(pValues=%p)\n",
        pValues);

    // Native variable declarations
    void* pValues_native;

    // Obtain native variable values
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return;
    }
    pValues_native = (void*)pValues_pointerData->getPointer(env);

    // Native function call
    nppsFree(pValues_native);

    // Write back native variable values
    if (!releasePointerData(env, pValues_pointerData, 0)) return;

}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_18uNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_8u(nValue=%c, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_8u(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_116sNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_16s(nValue=%d, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_16s(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_116scNative(JNIEnv *env, jclass cls, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSet_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_16sc(nValue=%p, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_16sc(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_132sNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_32s(nValue=%d, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_32s(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_132scNative(JNIEnv *env, jclass cls, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSet_32sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_32sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_32sc(nValue=%p, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp32sc nValue_native;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_32sc(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_132fNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_32f(nValue=%f, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_32f(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_132fcNative(JNIEnv *env, jclass cls, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSet_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_32fc(nValue=%p, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_32fc(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit long long integer, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_164sNative(JNIEnv *env, jclass cls, jlong nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_64s(nValue=%ld, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp64s nValue_native;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_64s(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit long long integer complex, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_164scNative(JNIEnv *env, jclass cls, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSet_64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_64sc(nValue=%p, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp64sc nValue_native;
    Npp64sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_64sc(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_164fNative(JNIEnv *env, jclass cls, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_64f(nValue=%lf, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_64f(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double complex, vector set method.
 * @param nValue Value used to initialize the vector pDst.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSet_164fcNative(JNIEnv *env, jclass cls, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSet_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSet_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSet_64fc(nValue=%p, pDst=%p, nLength=%d)\n",
        nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSet_64fc(nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_18uNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_8u(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_8u(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_116sNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_16s(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_16s(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_116scNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_16sc(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_16sc(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_132sNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_32s(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_32s(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_132scNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_32sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_32sc(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_32sc(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_132fNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_32f(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_32f(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_132fcNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_32fc(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_32fc(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit long long integer, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_164sNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_64s(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_64s(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_164scNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_64sc(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp64sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_64sc(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_164fNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_64f(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_64f(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double complex, vector zero method.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZero_164fcNative(JNIEnv *env, jclass cls, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsZero_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZero_64fc(pDst=%p, nLength=%d)\n",
        pDst, nLength);

    // Native variable declarations
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsZero_64fc(pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char, vector copy method
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_8u(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_8u(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_16s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_16s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_32s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_32s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_164sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_64s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_64s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex short, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_16sc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_16sc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex signed integer, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_132scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_32sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_32sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_32sc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_32sc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex float, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_32fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_32fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex signed integer, vector copy method.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_164scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_64sc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64sc* pSrc_native = NULL;
    Npp64sc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_64sc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCopy_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCopy_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCopy_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCopy_64fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCopy_64fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * @name Convert
 * Routines for converting the sample-data type of signals.
 *
 * </pre>
 */
//@{
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_18s16sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_8s16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_8s16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_8s16s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_8s16s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_18s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_8s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_8s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_8s32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_8s32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_18u32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_8u32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_8u32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_8u32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_8u32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116s8s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16s8s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16s8s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16s8s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp8s* pDst_native = NULL;
    Npp32u nLength_native;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nLength_native = (Npp32u)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16s8s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116s32sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16s32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16s32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16s32s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16s32s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16s32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16s32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116u32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16u32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16u32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16u32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16u32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s16sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s16s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s16s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s64fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f64fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164s64fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64s64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64s64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64s64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64s64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164f32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64f32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64f32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64f32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64f32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116s32f_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16s32f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16s32f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16s32f_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16s32f_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_116s64f_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_16s64f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_16s64f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_16s64f_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_16s64f_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s32f_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s32f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s32f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s32f_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s32f_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132s64f_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32s64f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32s64f_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32s64f_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32s64f_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f8s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f8s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f8s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f8s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp8s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f8s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f8u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f8u_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f8u_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f16s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f16u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f16u_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f16u_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_132f32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_32f32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_32f32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_32f32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_32f32s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64s32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64s32s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164f16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64f16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64f16s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164f32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64f32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64f32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64f32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64f32s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsConvert_164f64s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint eRoundMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsConvert_64f64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsConvert_64f64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsConvert_64f64s_Sfs(pSrc=%p, pDst=%p, nLength=%d, eRoundMode=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, eRoundMode, nScaleFactor);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode eRoundMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    eRoundMode_native = (NppRoundMode)eRoundMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsConvert_64f64s_Sfs(pSrc_native, pDst_native, nLength_native, eRoundMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // eRoundMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_16s(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_16s(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short signal threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_16s_I(pSrcDst=%p, nLength=%d, nLevel=%d, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_16s_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short complex number signal threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_16sc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_16sc(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_116sc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_16sc_I(pSrcDst=%p, nLength=%d, nLevel=%d, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_16sc_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_32f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_32f(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point signal threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_32f_I(pSrcDst=%p, nLength=%d, nLevel=%f, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_32f_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number signal threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_32fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_32fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point complex number signal threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_32fc_I(pSrcDst=%p, nLength=%d, nLevel=%f, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_32fc_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_64f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_64f(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point signal threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_64f_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_64f_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_64fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nRelOp=%d)\n",
        pSrc, pDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_64fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jint nRelOp)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_64fc_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nRelOp=%d)\n",
        pSrcDst, nLength, nLevel, nRelOp);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    NppCmpOp nRelOp_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nRelOp_native = (NppCmpOp)nRelOp;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_64fc_I(pSrcDst_native, nLength_native, nLevel_native, nRelOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nRelOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_16s(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_16s(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_16s_I(pSrcDst=%p, nLength=%d, nLevel=%d)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_16s_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_16sc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_16sc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_116sc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_16sc_I(pSrcDst=%p, nLength=%d, nLevel=%d)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_16sc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_32f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_32f(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_32f_I(pSrcDst=%p, nLength=%d, nLevel=%f)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_32f_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_32fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_32fc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_32fc_I(pSrcDst=%p, nLength=%d, nLevel=%f)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_32fc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_64f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_64f(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_64f_I(pSrcDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_64f_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LT_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LT_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_64fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_64fc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LT_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LT_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LT_64fc_I(pSrcDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LT_64fc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_16s(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_16s(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_16s_I(pSrcDst=%p, nLength=%d, nLevel=%d)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_16s_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_16sc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_16sc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_116sc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_16sc_I(pSrcDst=%p, nLength=%d, nLevel=%d)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_16sc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_32f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_32f(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_32f_I(pSrcDst=%p, nLength=%d, nLevel=%f)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_32f_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_32fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_32fc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_32fc_I(pSrcDst=%p, nLength=%d, nLevel=%f)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_32fc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_64f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_64f(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_64f_I(pSrcDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_64f_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GT_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GT_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_64fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrc, pDst, nLength, nLevel);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_64fc(pSrc_native, pDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GT_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GT_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GT_64fc_I(pSrcDst=%p, nLength=%d, nLevel=%lf)\n",
        pSrcDst, nLength, nLevel);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GT_64fc_I(pSrcDst_native, nLength_native, nLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_16s(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nValue=%d)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_16s(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_16s_I(pSrcDst=%p, nLength=%d, nLevel=%d, nValue=%d)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_16s_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_16sc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16sc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_16sc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_116sc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_16sc_I(pSrcDst=%p, nLength=%d, nLevel=%d, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16sc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_16sc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_32f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nValue=%f)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_32f(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_32f_I(pSrcDst=%p, nLength=%d, nLevel=%f, nValue=%f)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_32f_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_32fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32fc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_32fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_32fc_I(pSrcDst=%p, nLength=%d, nLevel=%f, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32fc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_32fc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jdouble nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_64f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nValue=%lf)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nValue_native = (Npp64f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_64f(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jdouble nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_64f_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nValue=%lf)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nValue_native = (Npp64f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_64f_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_LTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_LTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_64fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64fc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_64fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point complex number signal NPP_CMP_LESS threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1LTVal_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_LTVal_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_LTVal_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_LTVal_64fc_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64fc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_LTVal_64fc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_16s(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nValue=%d)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_16s(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jshort nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_16s_I(pSrcDst=%p, nLength=%d, nLevel=%d, nValue=%d)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16s nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    nValue_native = (Npp16s)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_16s_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_116scNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_16sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_16sc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%d, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16sc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_16sc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit in place signed short complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_116sc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jshort nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_16sc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_16sc_I(pSrcDst=%p, nLength=%d, nLevel=%d, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp16s nLevel_native;
    Npp16sc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp16s)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_16sc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_32f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nValue=%f)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_32f(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jfloat nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_32f_I(pSrcDst=%p, nLength=%d, nLevel=%f, nValue=%f)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    nValue_native = (Npp32f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_32f_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_32fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%f, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32fc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_32fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit in place floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_32fc_I(pSrcDst=%p, nLength=%d, nLevel=%f, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nLevel_native;
    Npp32fc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp32f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_32fc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jdouble nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_64f(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nValue=%lf)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64f nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nValue_native = (Npp64f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_64f(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jdouble nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_64f_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nValue=%lf)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64f nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    nValue_native = (Npp64f)nValue;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_64f_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsThreshold_GTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsThreshold_GTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_64fc(pSrc=%p, pDst=%p, nLength=%d, nLevel=%lf, nValue=%p)\n",
        pSrc, pDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64fc nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_64fc(pSrc_native, pDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit in place floating point complex number signal NPP_CMP_GREATER threshold with constant level.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nLevel Constant threshold value (real part only and must be greater than 0) to be used to limit each signal sample
 * @param nValue Constant value to replace source value when threshold test is true.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsThreshold_1GTVal_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jdouble nLevel, jobject nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsThreshold_GTVal_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nLevel is primitive
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsThreshold_GTVal_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsThreshold_GTVal_64fc_I(pSrcDst=%p, nLength=%d, nLevel=%lf, nValue=%p)\n",
        pSrcDst, nLength, nLevel, nValue);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp64f nLevel_native;
    Npp64fc nValue_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nLevel_native = (Npp64f)nLevel;
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppsThreshold_GTVal_64fc_I(pSrcDst_native, nLength_native, nLevel_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nLevel is primitive
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal add constant,
 * scale, then clamp to saturated value
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_18u_1ISfsNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_8u_ISfs(nValue=%c, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_8u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned charvector add constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_8u_Sfs(pSrc=%p, nValue=%c, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_8u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116u_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16u_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short vector add constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16u_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116s_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal add constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal add constant,
 * scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_16sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_16sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal add constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132s_1ISfsNative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integersignal add constant and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) in place signal
 * add constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc nValue_native;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) signal add constant
 * and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc nValue_native;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal add constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal add constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) in
 * place signal add constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_32fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_32fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_164f_1INative(JNIEnv *env, jclass cls, jdouble nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_64f_I(nValue=%lf, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_64f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating pointsignal add constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_64f(pSrc=%p, nValue=%lf, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_64f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) in
 * place signal add constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_164fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAddC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_64fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_64fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) signal
 * add constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be added to each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddC_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsAddC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddC_64fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddC_64fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProductC_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAddProductC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProductC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProductC_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddProductC_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal times constant,
 * scale, then clamp to saturated value
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_18u_1ISfsNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_8u_ISfs(nValue=%c, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_8u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_8u_Sfs(pSrc=%p, nValue=%c, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_8u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116u_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16u_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal times constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16u_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal times constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116s_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal times constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary)signal times constant, 
 * scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_16sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_16sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal times constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132s_1ISfsNative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal times constant and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) in place signal
 * times constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc nValue_native;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) signal times constant
 * and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc nValue_native;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal times constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_1Low_132f16sNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_Low_32f16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_Low_32f16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_Low_32f16s(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_Low_32f16s(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal times constant with output converted to 16-bit signed integer
 * with scaling and saturation of output result.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132f16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_32f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_32f16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32f16s_Sfs(pSrc=%p, nValue=%f, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32f16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) signal
 * times constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_32fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_32fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point, in place signal times constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength Length of the vectors, number of items.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_164f_1INative(JNIEnv *env, jclass cls, jdouble nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_64f_I(nValue=%lf, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_64f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal times constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_64f(pSrc=%p, nValue=%lf, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_64f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal times constant with in place conversion to 64-bit signed integer
 * and with scaling and saturation of output result.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_164f64s_1ISfsNative(JNIEnv *env, jclass cls, jdouble nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_64f64s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_64f64s_ISfs(nValue=%lf, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMulC_64f64s_ISfs(nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) in
 * place signal times constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_164fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMulC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_64fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_64fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) signal
 * times constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be multiplied by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMulC_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMulC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsMulC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMulC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMulC_64fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMulC_64fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal subtract constant,
 * scale, then clamp to saturated value
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_18u_1ISfsNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_8u_ISfs(nValue=%c, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_8u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal subtract constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_8u_Sfs(pSrc=%p, nValue=%c, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_8u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal subtract constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116u_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16u_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16u_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal subtract constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116s_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal subtract constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal subtract constant, 
 * scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal subtract constant,
 * scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_16sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_16sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal subtract constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132s_1ISfsNative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal subtract constant and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) in place signal
 * subtract constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc nValue_native;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary)signal subtract constant
 * and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc nValue_native;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal subtract constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal subtract constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) in
 * place signal subtract constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) signal
 * subtract constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_32fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_32fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point, in place signal subtract constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength Length of the vectors, number of items.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_164f_1INative(JNIEnv *env, jclass cls, jdouble nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_64f_I(nValue=%lf, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_64f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal subtract constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_64f(pSrc=%p, nValue=%lf, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_64f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) in
 * place signal subtract constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_164fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_64fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_64fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) signal
 * subtract constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be subtracted from each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubC_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubC_64fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubC_64fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal subtract from constant,
 * scale, then clamp to saturated value
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_18u_1ISfsNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_8u_ISfs(nValue=%c, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_8u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal subtract from constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_8u_Sfs(pSrc=%p, nValue=%c, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_8u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal subtract from constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116u_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16u_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal subtract from constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16u_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal subtract from constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116s_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal subtract from constant, 
 * scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal subtract from constant,
 * scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_16sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_16sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal subtract from constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132s_1ISfsNative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integersignal subtract from constant and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) in place signal
 * subtract from constant and scale.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc nValue_native;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer complex number (32 bit real, 32 bit imaginary) signal subtract from constant
 * and scale.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc nValue_native;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal subtract from constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal subtract from constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) in
 * place signal subtract from constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) signal
 * subtract from constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_32fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_32fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point, in place signal subtract from constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength Length of the vectors, number of items.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_164f_1INative(JNIEnv *env, jclass cls, jdouble nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_64f_I(nValue=%lf, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_64f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal subtract from constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_64f(pSrc=%p, nValue=%lf, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_64f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) in
 * place signal subtract from constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_164fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSubCRev_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_64fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_64fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) signal
 * subtract from constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value each vector element is to be subtracted from
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSubCRev_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSubCRev_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsSubCRev_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSubCRev_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSubCRev_64fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSubCRev_64fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal divided by constant,
 * scale, then clamp to saturated value
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_18u_1ISfsNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_8u_ISfs(nValue=%c, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_8u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal divided by constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_8u_Sfs(pSrc=%p, nValue=%c, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_8u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal divided by constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116u_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16u_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16u_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal divided by constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16u_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16u_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal divided by constant, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116s_1ISfsNative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16s_ISfs(nValue=%d, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16s_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal divided by constant, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16s_Sfs(pSrc=%p, nValue=%d, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16s_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary)signal divided by constant, 
 * scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16sc_ISfs(nValue=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        nValue, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc nValue_native;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16sc_ISfs(nValue_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer complex number (16 bit real, 16 bit imaginary) signal divided by constant,
 * scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_16sc_Sfs(pSrc=%p, nValue=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, nValue, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc nValue_native;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDivC_16sc_Sfs(pSrc_native, nValue_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal divided by constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal divided by constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) in
 * place signal divided by constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_132fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_32fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc nValue_native;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_32fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex number (32 bit real, 32 bit imaginary) signal
 * divided by constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_32fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc nValue_native;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_32fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place signal divided by constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength Length of the vectors, number of items.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_164f_1INative(JNIEnv *env, jclass cls, jdouble nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_64f_I(nValue=%lf, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64f nValue_native;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp64f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_64f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal divided by constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jdouble nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_64f(pSrc=%p, nValue=%lf, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f nValue_native;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp64f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_64f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) in
 * place signal divided by constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_164fc_1INative(JNIEnv *env, jclass cls, jobject nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivC_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_64fc_I(nValue=%p, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc nValue_native;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_64fc_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point complex number (64 bit real, 64 bit imaginary) signal
 * divided by constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided into each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivC_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nValue' is null for nppsDivC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivC_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivC_64fc(pSrc=%p, nValue=%p, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc nValue_native;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nValue, nValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivC_64fc(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nValue_native, nValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place constant divided by signal, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided by each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivCRev_116u_1INative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivCRev_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivCRev_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivCRev_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal divided by constant, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivCRev_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivCRev_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivCRev_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivCRev_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivCRev_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place constant divided by signal.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be divided by each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivCRev_132f_1INative(JNIEnv *env, jclass cls, jfloat nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDivCRev_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivCRev_32f_I(nValue=%f, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivCRev_32f_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point constant divided by signal.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be divided by each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDivCRev_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jfloat nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDivCRev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDivCRev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDivCRev_32f(pSrc=%p, nValue=%f, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDivCRev_32f(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116sNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned int signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    Npp32u* pSrc2_native = NULL;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_64f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_64f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal add signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_64fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_64fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal add signal with 16-bit unsigned result,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_18u16uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_8u16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_8u16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal add signal with 32-bit floating point result,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_8u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_8u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_164s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_64s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrc1_native = NULL;
    Npp64s* pSrc2_native = NULL;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_64s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed complex short add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed complex integer add signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be added to signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAdd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAdd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAdd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal add signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal add signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place signal add signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_164f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_64f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_64f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point in place signal add signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point in place signal add signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_64fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_64fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16/32-bit signed short in place signal add signal with 32-bit signed integer results,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116s32s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_16s32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_16s32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s32s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s32s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal add signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_8u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_8u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal add signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal add signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal add signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short in place signal add signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_16sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_16sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex signed integer in place signal add signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAdd_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAdd_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAdd_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAdd_32sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAdd_32sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_64f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_64f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_64fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_64fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal add product of source signal1 times source signal2 to destination signal,
 * with scaling, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed short signal add product of source signal1 times source signal2 to destination signal,
 * with scaling, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal add product of source signal1 times source signal2 to 32-bit signed integer destination signal,
 * with scaling, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAddProduct_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAddProduct_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAddProduct_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAddProduct_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAddProduct_16s32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsAddProduct_16s32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal times signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116sNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal times signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal times signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_64f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_64f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal times signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal times signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_64fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_64fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal times signal with 16-bit unsigned result,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_18u16uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_8u16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_8u16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal times signal with 32-bit floating point result,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal times 32-bit complex floating point signal with complex 32-bit floating point result,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132f32fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32f32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32f32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_8u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_8u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal time signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed complex short signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed complex integer signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal times 16-bit signed short signal, scale, then clamp to 16-bit signed saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116u16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16u16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16u16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16u16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16u16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16u16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal times signal, scale, then clamp to 32-bit signed saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal times 32-bit complex signed integer signal, scale, then clamp to 32-bit complex integer saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132s32sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32s32sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32s32sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal times signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_1Low_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsMul_Low_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsMul_Low_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsMul_Low_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_Low_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_Low_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal times signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal times signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place signal times signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_164f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_64f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_64f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point in place signal times signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point in place signal times signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_64fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_64fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point in place signal times 32-bit floating point signal,
 * then clamp to 32-bit complex floating point saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132f32fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32f32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32f32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32f32fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMul_32f32fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal times signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_8u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_8u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal times signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal times signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal times signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short in place signal times signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_16sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_16sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex signed integer in place signal times signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex signed integer in place signal times 32-bit signed integer signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMul_132s32sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMul_32s32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMul_32s32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMul_32s32sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsMul_32s32sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116sNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16s(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_16s(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_64f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_64f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_64fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_64fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal subtract 16-bit signed short signal,
 * then clamp and convert to 32-bit floating point saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16s32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_16s32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_8u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_8u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed complex short signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed complex integer signal subtract signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsSub_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsSub_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSub_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_32sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_16s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_164f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_64f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_64f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point in place signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_32fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point in place signal subtract signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_64fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSub_64fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal subtract signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_8u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_8u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal subtract signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal subtract signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal subtract signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_32s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_16sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_16sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex signed integer in place signal subtract signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSub_132sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSub_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSub_32sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSub_32sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSub_32sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_8u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_8u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed complex short signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16sc_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16sc_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal divided by 16-bit signed short signal, scale, then clamp to 16-bit signed short saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32s16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32s16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal divide signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal divide signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_64f(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_64f(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal divide signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal divide signal,
 * then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_64fc(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_64fc(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal divide signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_8u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_8u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal divide signal, with scaling,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal divide signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short in place signal divide signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_16sc_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_16sc_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place signal divide signal, with scaling, 
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place signal divide signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place signal divide signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_164f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_64f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_64f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point in place signal divide signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_32fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_32fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point in place signal divide signal,
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_64fc_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsDiv_64fc_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @defgroup signal_divround Div_Round
 * Sample by sample division of the samples of two signals with rounding.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal divide signal, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_Round_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_Round_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_Round_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_8u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_8u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal divide signal, scale, round, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_Round_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_Round_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_Round_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_16u_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_16u_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal divide signal, scale, round, then clamp to saturated value.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDiv_Round_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDiv_Round_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsDiv_Round_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_16s_Sfs(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc1, pSrc2, pDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_16s_Sfs(pSrc1_native, pSrc2_native, pDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal divide signal, with scaling, rounding
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_Round_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_Round_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_8u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_8u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal divide signal, with scaling, rounding
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_Round_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_Round_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_16u_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_16u_ISfs(pSrc_native, pSrcDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal divide signal, with scaling, rounding
 * then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * @param nLength \ref length_specification.
 * @param nRndMode various rounding modes.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDiv_1Round_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength, jint nRndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsDiv_Round_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsDiv_Round_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDiv_Round_16s_ISfs(pSrc=%p, pSrcDst=%p, nLength=%d, nRndMode=%d, nScaleFactor=%d)\n",
        pSrc, pSrcDst, nLength, nRndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    NppRoundMode nRndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nRndMode_native = (NppRoundMode)nRndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsDiv_Round_16s_ISfs(pSrc_native, pSrcDst_native, nLength_native, nRndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nRndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal absolute value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_16s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_16s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal absolute value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_32s(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_32s(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal absolute value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAbs_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAbs_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal absolute value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAbs_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAbs_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal absolute value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_116s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAbs_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_16s_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_16s_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal absolute value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_132s_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAbs_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_32s_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_32s_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAbs_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal absolute value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAbs_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAbs_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAbs_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAbs_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal squared.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal squared.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_32fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_32fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal squared.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_64fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_64fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal squared.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal squared.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal squared.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_32fc_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_32fc_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal squared.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_64fc_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqr_64fc_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal squared, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_8u_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_8u_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal squared, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16u_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16u_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short signal squared, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqr_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqr_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16sc_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16sc_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal squared, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_8u_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_8u_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal squared, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16u_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16u_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal squared, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short signal squared, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqr_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqr_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqr_16sc_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqr_16sc_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @defgroup signal_sqrt Sqrt
 * Square root of each sample of a signal.
 */
///@{
/**
 * <pre>
 * 32-bit floating point signal square root.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal square root.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal square root.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_32fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_32fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal square root.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64fc(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64fc(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal square root.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal square root.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal square root.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_132fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_32fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_32fc_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_32fc_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal square root.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164fc_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_64fc_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64fc_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64fc* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64fc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64fc_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal square root, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_18u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_8u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_8u_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_8u_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal square root, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116u_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_16u_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16u_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16u_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal square root, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short signal square root, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16sc_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16sc_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer signal square root, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal square root, scale, then clamp to 16-bit signed integer saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_132s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_32s16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_32s16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer signal square root, scale, then clamp to 16-bit signed integer saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSqrt_64s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSqrt_64s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64s16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64s16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char signal square root, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_18u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_8u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_8u_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_8u_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal square root, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116u_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_16u_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16u_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16u_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal square root, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short signal square root, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_116sc_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_16sc_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_16sc_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_16sc_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer signal square root, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSqrt_164s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsSqrt_64s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSqrt_64s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsSqrt_64s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_sqrt
/** @defgroup signal_cuberoot Cubrt
 * Cube root of each sample of a signal..
 */
///@{
/**
 * <pre>
 * 32-bit floating point signal cube root.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCubrt_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCubrt_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCubrt_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCubrt_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsCubrt_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal cube root, scale, then clamp to 16-bit signed integer saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCubrt_132s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCubrt_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsCubrt_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCubrt_32s16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsCubrt_32s16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal exponent.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsExp_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal exponent.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsExp_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal exponent with 64-bit floating point result.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_132f64fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_32f64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsExp_32f64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal exponent.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsExp_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsExp_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal exponent.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsExp_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsExp_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal exponent, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal exponent, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_32s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer signal exponent, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_164s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsExp_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsExp_64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_64s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrc_native = NULL;
    Npp64s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_64s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal exponent, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsExp_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_16s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_16s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal exponent, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsExp_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_32s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_32s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit signed integer signal exponent, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsExp_164s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsExp_64s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsExp_64s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp64s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsExp_64s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLn_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLn_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal natural logarithm with 32-bit floating point result.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_164f32fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_64f32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_64f32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_64f32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLn_64f32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal natural logarithm.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLn_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLn_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal natural logarithm.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLn_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLn_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal natural logarithm, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsLn_16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal natural logarithm, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsLn_32s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal natural logarithm, scale, then clamp to 16-bit signed short saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_132s16s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLn_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLn_32s16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_32s16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsLn_32s16s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal natural logarithm, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_116s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLn_16s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_16s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsLn_16s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal natural logarithm, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLn_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLn_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLn_32s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsLn_32s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal 10 times base 10 logarithm, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_npps10Log10_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for npps10Log10_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for npps10Log10_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing npps10Log10_32s_Sfs(pSrc=%p, pDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = npps10Log10_32s_Sfs(pSrc_native, pDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal 10 times base 10 logarithm, scale, then clamp to saturated value.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_npps10Log10_132s_1ISfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for npps10Log10_32s_ISfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing npps10Log10_32s_ISfs(pSrcDst=%p, nLength=%d, nScaleFactor=%d)\n",
        pSrcDst, nLength, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = npps10Log10_32s_ISfs(pSrcDst_native, nLength_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for 32f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLnGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumLnGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLnGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumLnGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal sum natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDst Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLn_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSumLn_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSumLn_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSumLn_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLn_32f(pSrc=%p, nLength=%d, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSumLn_32f(pSrc_native, nLength_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for 64f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLnGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumLnGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLnGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumLnGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal sum natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDst Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLn_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSumLn_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSumLn_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSumLn_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLn_64f(pSrc=%p, nLength=%d, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSumLn_64f(pSrc_native, nLength_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for 32f64f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_32f64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLnGetBufferSize_132f64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumLnGetBufferSize_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLnGetBufferSize_32f64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumLnGetBufferSize_32f64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit flaoting point input, 64-bit floating point output signal sum natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDst Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLn_132f64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSumLn_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSumLn_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSumLn_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLn_32f64f(pSrc=%p, nLength=%d, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSumLn_32f64f(pSrc_native, nLength_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for 16s32f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLnGetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumLnGetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLnGetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumLnGetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer input, 32-bit floating point output signal sum natural logarithm.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDst Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumLn_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pDst, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSumLn_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsSumLn_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSumLn_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumLn_16s32f(pSrc=%p, nLength=%d, pDst=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pDst, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pDst_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSumLn_16s32f(pSrc_native, nLength_native, pDst_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_sumln
/** @defgroup signal_inversetan Arctan
 * Inverse tangent of each sample of a signal.
 */
///@{
/**
 * <pre>
 * 32-bit floating point signal inverse tangent.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsArctan_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsArctan_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsArctan_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsArctan_32f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsArctan_32f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal inverse tangent.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsArctan_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsArctan_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsArctan_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsArctan_64f(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsArctan_64f(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal inverse tangent.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsArctan_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsArctan_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsArctan_32f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsArctan_32f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal inverse tangent.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsArctan_164f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsArctan_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsArctan_64f_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsArctan_64f_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_inversetan
/** @defgroup signal_normalize Normalize
 * Normalize each sample of a real or complex signal using offset and division operations.
 */
///@{
/**
 * <pre>
 * 32-bit floating point signal normalize.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jfloat vSub, jfloat vDiv)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_32f(pSrc=%p, pDst=%p, nLength=%d, vSub=%f, vDiv=%f)\n",
        pSrc, pDst, nLength, vSub, vDiv);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pDst_native = NULL;
    int nLength_native = 0;
    Npp32f vSub_native;
    Npp32f vDiv_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    vSub_native = (Npp32f)vSub;
    vDiv_native = (Npp32f)vDiv;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_32f(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex floating point signal normalize.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jobject vSub, jfloat vDiv)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (vSub == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'vSub' is null for nppsNormalize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // vDiv is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_32fc(pSrc=%p, pDst=%p, nLength=%d, vSub=%p, vDiv=%f)\n",
        pSrc, pDst, nLength, vSub, vDiv);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp32fc vSub_native;
    Npp32f vDiv_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    if (!initNative(env, vSub, vSub_native)) return JNPP_STATUS_INTERNAL_ERROR;
    vDiv_native = (Npp32f)vDiv;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_32fc(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releaseNative(env, vSub_native, vSub)) return JNPP_STATUS_INTERNAL_ERROR;
    // vDiv is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point signal normalize.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jdouble vSub, jdouble vDiv)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_64f(pSrc=%p, pDst=%p, nLength=%d, vSub=%lf, vDiv=%lf)\n",
        pSrc, pDst, nLength, vSub, vDiv);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pDst_native = NULL;
    int nLength_native = 0;
    Npp64f vSub_native;
    Npp64f vDiv_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64f*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    vSub_native = (Npp64f)vSub;
    vDiv_native = (Npp64f)vDiv;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_64f(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit complex floating point signal normalize.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jobject vSub, jdouble vDiv)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (vSub == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'vSub' is null for nppsNormalize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // vDiv is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_64fc(pSrc=%p, pDst=%p, nLength=%d, vSub=%p, vDiv=%lf)\n",
        pSrc, pDst, nLength, vSub, vDiv);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    Npp64fc* pDst_native = NULL;
    int nLength_native = 0;
    Npp64fc vSub_native;
    Npp64f vDiv_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp64fc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    if (!initNative(env, vSub, vSub_native)) return JNPP_STATUS_INTERNAL_ERROR;
    vDiv_native = (Npp64f)vDiv;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_64fc(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releaseNative(env, vSub_native, vSub)) return JNPP_STATUS_INTERNAL_ERROR;
    // vDiv is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal normalize, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jshort vSub, jint vDiv, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_16s_Sfs(pSrc=%p, pDst=%p, nLength=%d, vSub=%d, vDiv=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, vSub, vDiv, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;
    Npp16s vSub_native;
    int vDiv_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    vSub_native = (Npp16s)vSub;
    vDiv_native = (int)vDiv;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_16s_Sfs(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // vSub is primitive
    // vDiv is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex signed short signal normalize, scale, then clamp to saturated value.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @param vSub value subtracted from each signal element before division
 * @param vDiv divisor of post-subtracted signal element dividend
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormalize_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jobject vSub, jint vDiv, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNormalize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNormalize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (vSub == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'vSub' is null for nppsNormalize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // vDiv is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormalize_16sc_Sfs(pSrc=%p, pDst=%p, nLength=%d, vSub=%p, vDiv=%d, nScaleFactor=%d)\n",
        pSrc, pDst, nLength, vSub, vDiv, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nLength_native = 0;
    Npp16sc vSub_native;
    int vDiv_native = 0;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    if (!initNative(env, vSub, vSub_native)) return JNPP_STATUS_INTERNAL_ERROR;
    vDiv_native = (int)vDiv;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppsNormalize_16sc_Sfs(pSrc_native, pDst_native, nLength_native, vSub_native, vDiv_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releaseNative(env, vSub_native, vSub)) return JNPP_STATUS_INTERNAL_ERROR;
    // vDiv is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_normalize
/** @defgroup signal_cauchy Cauchy, CauchyD, and CauchyDD2
 * Determine Cauchy robust error function and its first and second derivatives for each sample of a signal.
 */
///@{
/**
 * <pre>
 * 32-bit floating point signal Cauchy error calculation.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nParam constant used in Cauchy formula
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCauchy_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nParam)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsCauchy_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nParam is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCauchy_32f_I(pSrcDst=%p, nLength=%d, nParam=%f)\n",
        pSrcDst, nLength, nParam);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nParam_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nParam_native = (Npp32f)nParam;

    // Native function call
    NppStatus jniResult_native = nppsCauchy_32f_I(pSrcDst_native, nLength_native, nParam_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nParam is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal Cauchy first derivative.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @param nParam constant used in Cauchy formula
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCauchyD_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength, jfloat nParam)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsCauchyD_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nParam is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCauchyD_32f_I(pSrcDst=%p, nLength=%d, nParam=%f)\n",
        pSrcDst, nLength, nParam);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;
    Npp32f nParam_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nParam_native = (Npp32f)nParam;

    // Native function call
    NppStatus jniResult_native = nppsCauchyD_32f_I(pSrcDst_native, nLength_native, nParam_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nParam is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point signal Cauchy first and second derivatives.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param pD2FVal \ref source_signal_pointer. This signal contains the second derivative
 *      of the source signal.
 * @param nLength \ref length_specification.
 * @param nParam constant used in Cauchy formula
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCauchyDD2_132f_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jobject pD2FVal, jint nLength, jfloat nParam)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsCauchyDD2_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pD2FVal == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pD2FVal' is null for nppsCauchyDD2_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    // nParam is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCauchyDD2_32f_I(pSrcDst=%p, pD2FVal=%p, nLength=%d, nParam=%f)\n",
        pSrcDst, pD2FVal, nLength, nParam);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    Npp32f* pD2FVal_native = NULL;
    int nLength_native = 0;
    Npp32f nParam_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    PointerData *pD2FVal_pointerData = initPointerData(env, pD2FVal);
    if (pD2FVal_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pD2FVal_native = (Npp32f*)pD2FVal_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    nParam_native = (Npp32f)nParam;

    // Native function call
    NppStatus jniResult_native = nppsCauchyDD2_32f_I(pSrcDst_native, pD2FVal_native, nLength_native, nParam_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pD2FVal_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    // nParam is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_cauchy
///@} signal_arithmetic_operations
/** @defgroup signal_logical_and_shift_operations Logical And Shift Operations
 *
 */
///@{
/** @defgroup
 */
///@{
/** @defgroup signal_andc AndC
 * Bitwise AND of a constant and each sample of a signal.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal and with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAndC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAndC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_8u(pSrc=%p, nValue=%c, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_8u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal and with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAndC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAndC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal and with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAndC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAndC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_32u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u nValue_native;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_32u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal and with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_18u_1INative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAndC_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_8u_I(nValue=%c, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_8u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal and with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_116u_1INative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAndC_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place signal and with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be anded with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAndC_132u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAndC_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAndC_32u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32u nValue_native;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAndC_32u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_andc
///@}
/** @defgroup signal_and And
 * Sample by sample bitwise AND of samples from two signals.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal and with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_18uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAnd_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAnd_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAnd_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_8u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_8u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal and with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_116uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAnd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAnd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAnd_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal and with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_132uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsAnd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsAnd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsAnd_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_32u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    Npp32u* pSrc2_native = NULL;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_32u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal and with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_18u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAnd_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAnd_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_8u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_8u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal and with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_116u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAnd_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAnd_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_16u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_16u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer in place signal and with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be anded with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsAnd_132u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsAnd_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsAnd_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsAnd_32u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsAnd_32u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_and
/** @defgroup signal_orc OrC
 * Bitwise OR of a constant and each sample of a signal.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOrC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOrC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_8u(pSrc=%p, nValue=%c, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_8u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOrC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOrC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOrC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOrC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_32u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u nValue_native;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_32u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_18u_1INative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOrC_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_8u_I(nValue=%c, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_8u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_116u_1INative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOrC_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place signal or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOrC_132u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOrC_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOrC_32u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32u nValue_native;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOrC_32u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_orc
/** @defgroup signal_or Or
 * Sample by sample bitwise OR of the samples from two signals.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_18uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsOr_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsOr_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOr_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_8u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_8u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_116uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsOr_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsOr_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOr_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_132uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsOr_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsOr_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsOr_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_32u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    Npp32u* pSrc2_native = NULL;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_32u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_18u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOr_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOr_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_8u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_8u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_116u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOr_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOr_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_16u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_16u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer in place signal or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsOr_132u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsOr_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsOr_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsOr_32u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsOr_32u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_or
/** @defgroup signal_xorc XorC
 * Bitwise XOR of a constant and each sample of a signal.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal exclusive or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jbyte nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXorC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXorC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_8u(pSrc=%p, nValue=%c, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_8u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal exclusive or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jshort nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXorC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXorC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal exclusive or with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXorC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXorC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_32u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u nValue_native;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nValue_native = (Npp32u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_32u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal exclusive or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_18u_1INative(JNIEnv *env, jclass cls, jbyte nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXorC_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_8u_I(nValue=%c, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_8u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal exclusive or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_116u_1INative(JNIEnv *env, jclass cls, jshort nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXorC_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place signal exclusive or with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be exclusive ored with each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXorC_132u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXorC_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXorC_32u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    Npp32u nValue_native;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32u)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXorC_32u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_xorc
/** @defgroup signal_xor Xor
 * Sample by sample bitwise XOR of the samples from two signals.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal exclusive or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_18uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsXor_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsXor_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXor_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_8u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    Npp8u* pSrc2_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_8u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal exclusive or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_116uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsXor_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsXor_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXor_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_16u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    Npp16u* pSrc2_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_16u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal exclusive or with signal.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_132uNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsXor_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsXor_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsXor_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_32u(pSrc1=%p, pSrc2=%p, pDst=%p, nLength=%d)\n",
        pSrc1, pSrc2, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    Npp32u* pSrc2_native = NULL;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_32u(pSrc1_native, pSrc2_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal exclusive or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_18u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXor_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXor_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_8u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_8u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal exclusive or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_116u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXor_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXor_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_16u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_16u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer in place signal exclusive or with signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer. signal2 elements to be exclusive ored with signal1 elements
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsXor_132u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsXor_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsXor_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsXor_32u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsXor_32u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_xor
/** @defgroup signal_not Not
 * Bitwise NOT of each sample of a signal.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char not signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNot_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNot_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_8u(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_8u(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short not signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNot_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNot_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_16u(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_16u(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer not signal.
 * @param pSrc \ref source_signal_pointer.
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNot_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsNot_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_32u(pSrc=%p, pDst=%p, nLength=%d)\n",
        pSrc, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_32u(pSrc_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place not signal.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_18u_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsNot_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_8u_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_8u_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place not signal.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_116u_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsNot_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_16u_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_16u_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place not signal.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNot_132u_1INative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsNot_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNot_32u_I(pSrcDst=%p, nLength=%d)\n",
        pSrcDst, nLength);

    // Native variable declarations
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsNot_32u_I(pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_not
/** @defgroup signal_lshiftc LShiftC
 * Left shifts the bits of each sample of a signal by a constant amount.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal left shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLShiftC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLShiftC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_8u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_8u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal left shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLShiftC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLShiftC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal left shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLShiftC_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLShiftC_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_16s(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nValue_native = 0;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_16s(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal left shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLShiftC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLShiftC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_32u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_32u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal left shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsLShiftC_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsLShiftC_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_32s(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nValue_native = 0;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_32s(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal left shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_18u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLShiftC_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_8u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_8u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal left shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_116u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLShiftC_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal left shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_116s_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLShiftC_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_16s_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_16s_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place signal left shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_132u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLShiftC_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_32u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_32u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed signed integer in place signal left shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to left shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsLShiftC_132s_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsLShiftC_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsLShiftC_32s_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsLShiftC_32s_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} signal_lshiftc
/** @defgroup signal_rshiftc RShiftC
 * Right shifts the bits of each sample of a signal by a constant amount.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned char signal right shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsRShiftC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsRShiftC_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_8u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp8u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_8u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short signal right shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsRShiftC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsRShiftC_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_16u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp16u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_16u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short signal right shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsRShiftC_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsRShiftC_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_16s(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nValue_native = 0;
    Npp16s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_16s(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned integer signal right shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsRShiftC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsRShiftC_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_32u(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nValue_native = 0;
    Npp32u* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_32u(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer signal right shift with constant.
 * @param pSrc \ref source_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param pDst \ref destination_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nValue, jobject pDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsRShiftC_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsRShiftC_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_32s(pSrc=%p, nValue=%d, pDst=%p, nLength=%d)\n",
        pSrc, nValue, pDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nValue_native = 0;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nValue_native = (int)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_32s(pSrc_native, nValue_native, pDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned char in place signal right shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_18u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsRShiftC_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_8u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_8u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short in place signal right shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_116u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsRShiftC_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_16u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_16u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short in place signal right shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_116s_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsRShiftC_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_16s_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_16s_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned signed integer in place signal right shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_132u_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsRShiftC_32u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_32u_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp32u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_32u_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed signed integer in place signal right shift with constant.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nValue Constant value to be used to right shift each vector element
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsRShiftC_132s_1INative(JNIEnv *env, jclass cls, jint nValue, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsRShiftC_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsRShiftC_32s_I(nValue=%d, pSrcDst=%p, nLength=%d)\n",
        nValue, pSrcDst, nLength);

    // Native variable declarations
    int nValue_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    nValue_native = (int)nValue;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsRShiftC_32s_I(nValue_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_18u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_8u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_8u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short integer in place min value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_116u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_16u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_16u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_116s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_16s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_16s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_132s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_32s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_32s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit floating point in place min value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinEvery_164f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinEvery_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMinEvery_64f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinEvery_64f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    Npp64f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp64f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMinEvery_64f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit in place max value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxEvery_18u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxEvery_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMaxEvery_8u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxEvery_8u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMaxEvery_8u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short integer in place max value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxEvery_116u_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxEvery_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMaxEvery_16u_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxEvery_16u_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMaxEvery_16u_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer in place max value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxEvery_116s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxEvery_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMaxEvery_16s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxEvery_16s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMaxEvery_16s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer in place max value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxEvery_132s_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxEvery_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMaxEvery_32s_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxEvery_32s_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMaxEvery_32s_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point in place max value for each pair of elements.
 * @param pSrc \ref source_signal_pointer.
 * @param pSrcDst \ref in_place_signal_pointer.
 * @param nLength \ref length_specification.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxEvery_132f_1INative(JNIEnv *env, jclass cls, jobject pSrc, jobject pSrcDst, jint nLength)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxEvery_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppsMaxEvery_32f_I");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxEvery_32f_I(pSrc=%p, pSrcDst=%p, nLength=%d)\n",
        pSrc, pSrcDst, nLength);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nLength_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;

    // Native function call
    NppStatus jniResult_native = nppsMaxEvery_32f_I(pSrc_native, pSrcDst_native, nLength_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_132fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_164fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_16s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_116s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_16s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_16s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_16sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_116sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_16sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_16sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_16sc32sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_116sc32sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_16sc32sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_16sc32sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_132s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsSum_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSumGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsSumGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSumGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsSumGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector sum method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_32f(pSrc=%p, nLength=%d, pSum=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pSum_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp32f*)pSum_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_32f(pSrc_native, nLength_native, pSum_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_32fc(pSrc=%p, nLength=%d, pSum=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32fc* pSum_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp32fc*)pSum_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_32fc(pSrc_native, nLength_native, pSum_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double vector sum method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_64f(pSrc=%p, nLength=%d, pSum=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pSum_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64f*)pSum_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_64f(pSrc_native, nLength_native, pSum_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double complex vector sum method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_64fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_64fc(pSrc=%p, nLength=%d, pSum=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64fc* pSum_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp64fc*)pSum_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_64fc(pSrc_native, nLength_native, pSum_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit short vector sum with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_16s_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_16s_Sfs(pSrc=%p, nLength=%d, pSum=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pSum_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp16s*)pSum_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_16s_Sfs(pSrc_native, nLength_native, pSum_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector sum with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_32s_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_32s_Sfs(pSrc=%p, nLength=%d, pSum=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pSum_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp32s*)pSum_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_32s_Sfs(pSrc_native, nLength_native, pSum_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit short complex vector sum with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_16sc_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_16sc_Sfs(pSrc=%p, nLength=%d, pSum=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16sc* pSum_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp16sc*)pSum_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_16sc_Sfs(pSrc_native, nLength_native, pSum_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit short complex vector sum (32bit int complex) with integer scaling
 * method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_16sc32sc_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_116sc32sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_16sc32sc_Sfs(pSrc=%p, nLength=%d, pSum=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32sc* pSum_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp32sc*)pSum_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_16sc32sc_Sfs(pSrc_native, nLength_native, pSum_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector sum (32bit) with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pSum Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsSumGetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsSum_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pSum, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsSum_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppsSum_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsSum_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsSum_16s32s_Sfs(pSrc=%p, nLength=%d, pSum=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pSum, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pSum_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pSum_pointerData = initPointerData(env, pSum);
    if (pSum_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSum_native = (Npp32s*)pSum_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsSum_16s32s_Sfs(pSrc_native, nLength_native, pSum_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pSum_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMax_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMax_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMax_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMax_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMax_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMax_16s(pSrc=%p, nLength=%d, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMax_16s(pSrc_native, nLength_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMax_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMax_32s(pSrc=%p, nLength=%d, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32s*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMax_32s(pSrc_native, nLength_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMax_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMax_32f(pSrc=%p, nLength=%d, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMax_32f(pSrc_native, nLength_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMax_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMax_64f(pSrc=%p, nLength=%d, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp64f*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMax_64f(pSrc_native, nLength_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxIndx_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxIndxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxIndxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxIndx_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxIndxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxIndxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxIndx_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndxGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxIndxGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndxGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxIndxGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndxGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxIndxGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndxGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxIndxGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector max index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pIndx Pointer to the index value of the first maximum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxIndxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndx_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndx_16s(pSrc=%p, nLength=%d, pMax=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMax_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxIndx_16s(pSrc_native, nLength_native, pMax_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector max index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pIndx Pointer to the index value of the first maximum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxIndxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndx_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndx_32s(pSrc=%p, nLength=%d, pMax=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMax_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32s*)pMax_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxIndx_32s(pSrc_native, nLength_native, pMax_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector max index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pIndx Pointer to the index value of the first maximum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxIndxGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndx_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndx_32f(pSrc=%p, nLength=%d, pMax=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMax_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxIndx_32f(pSrc_native, nLength_native, pMax_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector max index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMax Pointer to the output result.
 * @param pIndx Pointer to the index value of the first maximum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxIndxGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxIndx_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMax, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxIndx_64f(pSrc=%p, nLength=%d, pMax=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMax, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMax_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp64f*)pMax_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxIndx_64f(pSrc_native, nLength_native, pMax_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxAbs_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxAbsGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxAbs_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxAbsGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector max absolute method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMaxAbs Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxAbsGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbs_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMaxAbs, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMaxAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxAbs' is null for nppsMaxAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbs_16s(pSrc=%p, nLength=%d, pMaxAbs=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMaxAbs, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMaxAbs_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMaxAbs_pointerData = initPointerData(env, pMaxAbs);
    if (pMaxAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxAbs_native = (Npp16s*)pMaxAbs_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxAbs_16s(pSrc_native, nLength_native, pMaxAbs_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMaxAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector max absolute method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMaxAbs Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMaxAbsGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbs_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMaxAbs, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMaxAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxAbs' is null for nppsMaxAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbs_32s(pSrc=%p, nLength=%d, pMaxAbs=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMaxAbs, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMaxAbs_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMaxAbs_pointerData = initPointerData(env, pMaxAbs);
    if (pMaxAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxAbs_native = (Npp32s*)pMaxAbs_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxAbs_32s(pSrc_native, nLength_native, pMaxAbs_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMaxAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxAbsIndx_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsIndxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxAbsIndxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsIndxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsIndxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMaxAbsIndx_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsIndxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMaxAbsIndxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsIndxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsIndxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsIndx_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMaxAbs, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMaxAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxAbs' is null for nppsMaxAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsIndx_16s(pSrc=%p, nLength=%d, pMaxAbs=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMaxAbs, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMaxAbs_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMaxAbs_pointerData = initPointerData(env, pMaxAbs);
    if (pMaxAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxAbs_native = (Npp16s*)pMaxAbs_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsIndx_16s(pSrc_native, nLength_native, pMaxAbs_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMaxAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMaxAbsIndx_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMaxAbs, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMaxAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMaxAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxAbs' is null for nppsMaxAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMaxAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMaxAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMaxAbsIndx_32s(pSrc=%p, nLength=%d, pMaxAbs=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMaxAbs, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMaxAbs_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMaxAbs_pointerData = initPointerData(env, pMaxAbs);
    if (pMaxAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxAbs_native = (Npp32s*)pMaxAbs_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMaxAbsIndx_32s(pSrc_native, nLength_native, pMaxAbs_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMaxAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMin_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMin_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMin_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMin_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector min method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMin_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMin_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMin_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMin_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMin_16s(pSrc=%p, nLength=%d, pMin=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMin_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMin_16s(pSrc_native, nLength_native, pMin_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector min method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMin_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMin_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMin_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMin_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMin_32s(pSrc=%p, nLength=%d, pMin=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMin_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32s*)pMin_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMin_32s(pSrc_native, nLength_native, pMin_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector min method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMin_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMin_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMin_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMin_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMin_32f(pSrc=%p, nLength=%d, pMin=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMin_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMin_32f(pSrc_native, nLength_native, pMin_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit integer vector min method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMin_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMin_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMin_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMin_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMin_64f(pSrc=%p, nLength=%d, pMin=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMin_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp64f*)pMin_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMin_64f(pSrc_native, nLength_native, pMin_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinIndx_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinIndxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinIndxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinIndx_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinIndxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinIndxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinIndx_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndxGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinIndxGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndxGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinIndxGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndxGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinIndxGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndxGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinIndxGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector min index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pIndx Pointer to the index value of the first minimum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinIndxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndx_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndx_16s(pSrc=%p, nLength=%d, pMin=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMin_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinIndx_16s(pSrc_native, nLength_native, pMin_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector min index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pIndx Pointer to the index value of the first minimum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinIndxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndx_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndx_32s(pSrc=%p, nLength=%d, pMin=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMin_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32s*)pMin_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinIndx_32s(pSrc_native, nLength_native, pMin_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndx_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndx_32f(pSrc=%p, nLength=%d, pMin=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMin_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinIndx_32f(pSrc_native, nLength_native, pMin_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector min index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the output result.
 * @param pIndx Pointer to the index value of the first minimum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinIndxGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinIndx_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinIndx_64f(pSrc=%p, nLength=%d, pMin=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMin_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp64f*)pMin_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinIndx_64f(pSrc_native, nLength_native, pMin_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinAbs_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinAbsGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinAbsGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinAbs_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinAbsGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinAbsGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector min absolute method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMinAbs Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinAbsGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbs_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMinAbs, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMinAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinAbs' is null for nppsMinAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinAbs_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbs_16s(pSrc=%p, nLength=%d, pMinAbs=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMinAbs, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMinAbs_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMinAbs_pointerData = initPointerData(env, pMinAbs);
    if (pMinAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinAbs_native = (Npp16s*)pMinAbs_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinAbs_16s(pSrc_native, nLength_native, pMinAbs_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMinAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector min absolute method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMinAbs Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinAbsGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbs_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMinAbs, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMinAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinAbs' is null for nppsMinAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinAbs_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbs_32s(pSrc=%p, nLength=%d, pMinAbs=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMinAbs, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMinAbs_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMinAbs_pointerData = initPointerData(env, pMinAbs);
    if (pMinAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinAbs_native = (Npp32s*)pMinAbs_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinAbs_32s(pSrc_native, nLength_native, pMinAbs_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMinAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMinAbsIndx_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsIndxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinAbsIndxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsIndxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinAbsIndxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsIndxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinAbsIndxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsIndxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinAbsIndxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit integer vector min absolute index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMinAbs Pointer to the output result.
 * @param pIndx Pointer to the index value of the first minimum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinAbsIndxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsIndx_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMinAbs, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMinAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinAbs' is null for nppsMinAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinAbsIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsIndx_16s(pSrc=%p, nLength=%d, pMinAbs=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMinAbs, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMinAbs_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMinAbs_pointerData = initPointerData(env, pMinAbs);
    if (pMinAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinAbs_native = (Npp16s*)pMinAbs_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinAbsIndx_16s(pSrc_native, nLength_native, pMinAbs_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMinAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector min absolute index method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMinAbs Pointer to the output result.
 * @param pIndx Pointer to the index value of the first minimum element.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinAbsIndxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinAbsIndx_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMinAbs, jintArray pIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMinAbs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinAbs' is null for nppsMinAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pIndx' is null for nppsMinAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinAbsIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinAbsIndx_32s(pSrc=%p, nLength=%d, pMinAbs=%p, pIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMinAbs, pIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMinAbs_native = NULL;
    int* pIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMinAbs_pointerData = initPointerData(env, pMinAbs);
    if (pMinAbs_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinAbs_native = (Npp32s*)pMinAbs_pointerData->getPointer(env);
    PointerData *pIndx_pointerData = initPointerData(env, pIndx);
    if (pIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pIndx_native = (int*)pIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinAbsIndx_32s(pSrc_native, nLength_native, pMinAbs_native, pIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMinAbs_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@{                                         
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_132fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_164fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_16s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_116s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_16s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_16s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_132s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMean_16sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanGetBufferSize_116sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanGetBufferSize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanGetBufferSize_16sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanGetBufferSize_16sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector mean method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_32f(pSrc=%p, nLength=%d, pMean=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMean_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp32f*)pMean_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_32f(pSrc_native, nLength_native, pMean_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex vector mean method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_32fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_132fcNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_32fc(pSrc=%p, nLength=%d, pMean=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32fc* pMean_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp32fc*)pMean_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_32fc(pSrc_native, nLength_native, pMean_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double vector mean method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_64f(pSrc=%p, nLength=%d, pMean=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMean_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_64f(pSrc_native, nLength_native, pMean_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double complex vector mean method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_64fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_164fcNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_64fc(pSrc=%p, nLength=%d, pMean=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64fc* pMean_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64fc*)pMean_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_64fc(pSrc_native, nLength_native, pMean_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit short vector mean with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_16s_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_16s_Sfs(pSrc=%p, nLength=%d, pMean=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMean_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp16s*)pMean_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_16s_Sfs(pSrc_native, nLength_native, pMean_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit integer vector mean with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_32s_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_32s_Sfs(pSrc=%p, nLength=%d, pMean=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMean_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp32s*)pMean_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_32s_Sfs(pSrc_native, nLength_native, pMean_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit short complex vector mean with integer scaling method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanGetBufferSize_16sc_Sfs to determine the minium number of bytes required.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMean_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMean_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMean_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMean_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMean_16sc_Sfs(pSrc=%p, nLength=%d, pMean=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16sc* pMean_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp16sc*)pMean_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMean_16sc_Sfs(pSrc_native, nLength_native, pMean_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsStdDev_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDevGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsStdDevGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDevGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsStdDevGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsStdDev_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDevGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsStdDevGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDevGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsStdDevGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsStdDev_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDevGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsStdDevGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDevGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsStdDevGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsStdDev_16s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDevGetBufferSize_116s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsStdDevGetBufferSize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDevGetBufferSize_16s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsStdDevGetBufferSize_16s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDev_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pStdDev, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDev_32f(pSrc=%p, nLength=%d, pStdDev=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pStdDev, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pStdDev_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp32f*)pStdDev_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsStdDev_32f(pSrc_native, nLength_native, pStdDev_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector standard deviation method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pStdDev Pointer to the output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsStdDevGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDev_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pStdDev, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDev_64f(pSrc=%p, nLength=%d, pStdDev=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pStdDev, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pStdDev_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsStdDev_64f(pSrc_native, nLength_native, pStdDev_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDev_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pStdDev, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDev_16s32s_Sfs(pSrc=%p, nLength=%d, pStdDev=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pStdDev, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pStdDev_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp32s*)pStdDev_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsStdDev_16s32s_Sfs(pSrc_native, nLength_native, pStdDev_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit float vector standard deviation method (return value is also 16-bit)
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pStdDev Pointer to the output result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsStdDevGetBufferSize_16s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsStdDev_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pStdDev, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsStdDev_16s_Sfs(pSrc=%p, nLength=%d, pStdDev=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pStdDev, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pStdDev_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp16s*)pStdDev_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsStdDev_16s_Sfs(pSrc_native, nLength_native, pStdDev_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMeanStdDev_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDevGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanStdDevGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDevGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDevGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** host pointer */
/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMeanStdDev_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDevGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanStdDevGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDevGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDevGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMeanStdDev_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDevGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanStdDevGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDevGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDevGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device scratch buffer size (in bytes) for nppsMeanStdDev_16s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDevGetBufferSize_116s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMeanStdDevGetBufferSize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDevGetBufferSize_16s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDevGetBufferSize_16s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector mean and standard deviation method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output mean value.
 * @param pStdDev Pointer to the output standard deviation value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanStdDevGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDev_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pStdDev, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMeanStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMeanStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsMeanStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMeanStdDev_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDev_32f(pSrc=%p, nLength=%d, pMean=%p, pStdDev=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pStdDev, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMean_native = NULL;
    Npp32f* pStdDev_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp32f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp32f*)pStdDev_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDev_32f(pSrc_native, nLength_native, pMean_native, pStdDev_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector mean and standard deviation method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output mean value.
 * @param pStdDev Pointer to the output standard deviation value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanStdDevGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDev_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pStdDev, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMeanStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMeanStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsMeanStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMeanStdDev_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDev_64f(pSrc=%p, nLength=%d, pMean=%p, pStdDev=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pStdDev, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDev_64f(pSrc_native, nLength_native, pMean_native, pStdDev_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit float vector mean and standard deviation method (return values are 32-bit)
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output mean value.
 * @param pStdDev Pointer to the output standard deviation value.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanStdDevGetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDev_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pStdDev, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMeanStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMeanStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsMeanStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMeanStdDev_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDev_16s32s_Sfs(pSrc=%p, nLength=%d, pMean=%p, pStdDev=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pStdDev, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMean_native = NULL;
    Npp32s* pStdDev_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp32s*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp32s*)pStdDev_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDev_16s32s_Sfs(pSrc_native, nLength_native, pMean_native, pStdDev_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit float vector mean and standard deviation method (return values are also 16-bit)
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMean Pointer to the output mean value.
 * @param pStdDev Pointer to the output standard deviation value.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMeanStdDevGetBufferSize_16s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMeanStdDev_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMean, jobject pStdDev, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMeanStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppsMeanStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppsMeanStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMeanStdDev_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMeanStdDev_16s_Sfs(pSrc=%p, nLength=%d, pMean=%p, pStdDev=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMean, pStdDev, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMean_native = NULL;
    Npp16s* pStdDev_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp16s*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp16s*)pStdDev_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMeanStdDev_16s_Sfs(pSrc_native, nLength_native, pMean_native, pStdDev_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_8u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_18uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_8u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_8u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_16u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_116uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_16u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_16u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_32u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_132uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_32u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_32u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMax_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit char vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_8u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_8u(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp8u* pMin_native = NULL;
    Npp8u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_8u(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_16s(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMin_native = NULL;
    Npp16s* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_16s(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_16u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_16u(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16u* pMin_native = NULL;
    Npp16u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_16u(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned int vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_32u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_32u(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32u* pMin_native = NULL;
    Npp32u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_32u(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed int vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_32s(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMin_native = NULL;
    Npp32s* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32s*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32s*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_32s(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_32f(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMin_native = NULL;
    Npp32f* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_32f(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit double vector min and max method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMax Pointer to the max output result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMax_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMax_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMax_64f(pSrc=%p, nLength=%d, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMin_native = NULL;
    Npp64f* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp64f*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp64f*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMax_64f(pSrc_native, nLength_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_8u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_18uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_8u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_8u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_16s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_116sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_16s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_16s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_16u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_116uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_16u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_16u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_32u.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_132uNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_32u(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_32u(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsMinMaxIndx_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndxGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsMinMaxIndxGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndxGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndxGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit char vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_8u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_18uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_8u(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp8u* pMin_native = NULL;
    int* pMinIndx_native;
    Npp8u* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_8u(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_16s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_116sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_16s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_16s(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16s* pMin_native = NULL;
    int* pMinIndx_native;
    Npp16s* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16s*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16s*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_16s(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned short vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_16u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_116uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_16u(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp16u* pMin_native = NULL;
    int* pMinIndx_native;
    Npp16u* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp16u*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp16u*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_16u(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed short vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_32s(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pMin_native = NULL;
    int* pMinIndx_native;
    Npp32s* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32s*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32s*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_32s(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit unsigned short vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_32u to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_132uNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_32u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_32u(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32u* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32u* pMin_native = NULL;
    int* pMinIndx_native;
    Npp32u* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32u*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32u*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32u*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_32u(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_32f(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pMin_native = NULL;
    int* pMinIndx_native;
    Npp32f* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp32f*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp32f*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_32f(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector min and max with indices method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pMin Pointer to the min output result.
 * @param pMinIndx Pointer to the index of the first min value.
 * @param pMax Pointer to the max output result.
 * @param pMaxIndx Pointer to the index of the first max value.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsMinMaxIndxGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsMinMaxIndx_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pMin, jintArray pMinIndx, jobject pMax, jintArray pMaxIndx, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMinIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMinIndx' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMaxIndx == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMaxIndx' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsMinMaxIndx_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsMinMaxIndx_64f(pSrc=%p, nLength=%d, pMin=%p, pMinIndx=%p, pMax=%p, pMaxIndx=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pMin, pMinIndx, pMax, pMaxIndx, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pMin_native = NULL;
    int* pMinIndx_native;
    Npp64f* pMax_native = NULL;
    int* pMaxIndx_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp64f*)pMin_pointerData->getPointer(env);
    PointerData *pMinIndx_pointerData = initPointerData(env, pMinIndx);
    if (pMinIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMinIndx_native = (int*)pMinIndx_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp64f*)pMax_pointerData->getPointer(env);
    PointerData *pMaxIndx_pointerData = initPointerData(env, pMaxIndx);
    if (pMaxIndx_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMaxIndx_native = (int*)pMaxIndx_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsMinMaxIndx_64f(pSrc_native, nLength_native, pMin_native, pMinIndx_native, pMax_native, pMaxIndx_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMinIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMaxIndx_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector C norm method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector C norm method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector C norm method, return value is 32-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_16s32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_16s32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_32fc32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_132fc32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_32fc32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_32fc32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex vector C norm method, return value is 32-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_32fc32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_132fc32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_32fc32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_32fc32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex vector C norm method, return value is 64-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_64fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_64fc64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_64fc64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_Inf_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormInfGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormInfGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormInfGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormInfGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector C norm method, return value is 32-bit signed integer.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormInfGetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1Inf_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_Inf_16s32s_Sfs(pSrc=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_Inf_16s32s_Sfs(pSrc_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector L1 norm method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L1 norm method, return value is 32-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the L1 norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_16s32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_16s32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_32fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_132fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_32fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_32fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex vector L1 norm method, return value is 64-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_32fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_132fc64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_32fc64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_32fc64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex vector L1 norm method, return value is 64-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_64fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_64fc64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_64fc64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L1 norm method, return value is 32-bit signed integer.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_16s32s_Sfs(pSrc=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_16s32s_Sfs(pSrc_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L1_16s64s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL1GetBufferSize_116s64s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL1GetBufferSize_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL1GetBufferSize_16s64s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL1GetBufferSize_16s64s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L1 norm method, return value is 64-bit signed integer.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL1GetBufferSize_16s64s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L1_116s64s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L1_16s64s_Sfs(pSrc=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L1_16s64s_Sfs(pSrc_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float vector L2 norm method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float vector L2 norm method
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_164fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L2 norm method, return value is 32-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_16s32f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_16s32f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_32fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_132fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_32fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_32fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex vector L2 norm method, return value is 64-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_32fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_132fc64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_32fc64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_32fc64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex vector L2 norm method, return value is 64-bit float.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_64fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_64fc64f(pSrc=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp64fc*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_64fc64f(pSrc_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2GetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2GetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2GetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2GetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L2 norm method, return value is 32-bit signed integer.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2GetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2_16s32s_Sfs(pSrc=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2_16s32s_Sfs(pSrc_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNorm_L2Sqr_16s64s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormL2SqrGetBufferSize_116s64s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormL2SqrGetBufferSize_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormL2SqrGetBufferSize_16s64s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormL2SqrGetBufferSize_16s64s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer vector L2 Square norm method, return value is 64-bit signed integer.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormL2SqrGetBufferSize_16s64s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNorm_1L2Sqr_116s64s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsNorm_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNorm_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNorm_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNorm_L2Sqr_16s64s_Sfs(pSrc=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp64s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNorm_L2Sqr_16s64s_Sfs(pSrc_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_Inf_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float C norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_Inf_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float C norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_Inf_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer C norm method on two vectors' difference, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_16s32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_16s32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_Inf_32fc32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_132fc32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_32fc32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_32fc32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex C norm method on two vectors' difference, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_32fc32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_132fc32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_32fc32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_32fc32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_32fc32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_Inf_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex C norm method on two vectors' difference, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_64fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_64fc64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_64fc64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffInfGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffInfGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffInfGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffInfGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer C norm method on two vectors' difference, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffInfGetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1Inf_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_Inf_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_Inf_16s32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_Inf_16s32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float L1 norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float L1 norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the L1 norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_16s32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_16s32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_32fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_132fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_32fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_32fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex L1 norm method on two vectors' difference, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_32fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_132fc64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_32fc64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_32fc64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_64fc64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_64fc64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer..
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_16s32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_16s32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L1_16s64s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL1GetBufferSize_116s64s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL1GetBufferSize_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL1GetBufferSize_16s64s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL1GetBufferSize_16s64s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 64-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL1GetBufferSize_16s64s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L1_116s64s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L1_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L1_16s64s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L1_16s64s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @defgroup signal_L2_norm_diff L2 Norm Diff
 */
///@{                                         
/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float L2 norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float L2 norm method on two vectors' difference
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L2 norm method on two vectors' difference, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_16s32f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_16s32f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_32fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_132fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_32fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_32fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex L2 norm method on two vectors' difference, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_32fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_132fc64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_32fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_32fc64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_32fc64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_64fc64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_164fc64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_64fc64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_64fc64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex L2 norm method on two vectors' difference, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_64fc64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_164fc64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_64fc64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_64fc64f(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pNorm_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64f*)pNorm_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_64fc64f(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2GetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2GetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2GetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2GetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L2 norm method on two vectors' difference, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2GetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2_16s32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp32s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2_16s32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsNormDiff_L2Sqr_16s64s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiffL2SqrGetBufferSize_116s64s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsNormDiffL2SqrGetBufferSize_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiffL2SqrGetBufferSize_16s64s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsNormDiffL2SqrGetBufferSize_16s64s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer L2 Square norm method on two vectors' difference, return value is 64-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pNorm Pointer to the norm result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsNormDiffL2SqrGetBufferSize_16s64s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsNormDiff_1L2Sqr_116s64s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pNorm, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsNormDiff_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsNormDiff_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pNorm == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pNorm' is null for nppsNormDiff_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsNormDiff_L2Sqr_16s64s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsNormDiff_L2Sqr_16s64s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pNorm=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pNorm, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64s* pNorm_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pNorm_pointerData = initPointerData(env, pNorm);
    if (pNorm_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pNorm_native = (Npp64s*)pNorm_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsNormDiff_L2Sqr_16s64s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pNorm_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pNorm_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float dot product method, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32f(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32f(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float complex dot product method, return value is 32-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32f32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132f32fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32f32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32f32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float and 32-bit float complex dot product method, return value is 32-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32f32fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132f32fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32f32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32f32fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32f32fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32f64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132f64fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32f64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32f64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float dot product method, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32f64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132f64fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32f64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32f64f(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32f64f(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32fc64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132fc64fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32fc64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32fc64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132fc64fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32fc64fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32fc64fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32f32fc64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132f32fc64fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32f32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32f32fc64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32f32fc64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float and 32-bit float complex dot product method, return value is 64-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32f32fc64fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132f32fc64fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32f32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32f32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32f32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32f32fc64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32f32fc64fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    Npp32fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32f32fc64fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_64f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_164fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_64f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_64f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float dot product method, return value is 64-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_64f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_164fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_64f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_64f(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64f* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64f*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_64f(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_164fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float complex dot product method, return value is 64-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_64fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_164fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_64fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp64fc* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64fc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_64fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_64f64fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_164f64fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_64f64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_64f64fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_64f64fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 64-bit float and 64-bit float complex dot product method, return value is 64-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_64f64fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_164f64fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_64f64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_64f64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_64f64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_64f64fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_64f64fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp64f* pSrc1_native = NULL;
    Npp64fc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp64f*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp64fc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_64f64fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s64s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s64sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s64s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s64s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer dot product method, return value is 64-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s64s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s64sNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s64s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s64s(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64s* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64s*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s64s(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16sc64sc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116sc64scNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16sc64sc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16sc64sc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer complex dot product method, return value is 64-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16sc64sc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116sc64scNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16sc64sc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64sc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64sc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16sc64sc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s16sc64sc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s16sc64scNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s16sc64sc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s16sc64sc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer and 16-bit signed short integer short dot product method, return value is 64-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s16sc64sc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s16sc64scNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s16sc64sc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s16sc64sc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp64sc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp64sc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s16sc64sc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer dot product method, return value is 32-bit float.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s32f(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32f* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32f*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s32f(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16sc32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116sc32fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16sc32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16sc32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer complex dot product method, return value is 32-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16sc32fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116sc32fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16sc32fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16sc32fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s16sc32fc.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s16sc32fcNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s16sc32fc(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s16sc32fc(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer and 16-bit signed short integer complex dot product method, return value is 32-bit float complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s16sc32fc to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s16sc32fcNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s16sc32fc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s16sc32fc(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32fc* pDp_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32fc*)pDp_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s16sc32fc(pSrc1_native, pSrc2_native, nLength_native, pDp_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer dot product method, return value is 16-bit signed short integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp16s* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp16s*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer complex dot product method, return value is 16-bit signed short integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp16sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp16sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer dot product method, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32s*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed integer complex dot product method, return value is 32-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer dot product method, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result. 
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32s*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s16sc32sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s16sc32sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s16sc32sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s16sc32sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer and 16-bit signed short integer complex dot product method, return value is 32-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s16sc32sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s16sc32sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s16sc32sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s16sc32sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s32s32s_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s32s32s_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s32s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s32s32s_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s32s32s_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer and 32-bit signed integer dot product method, return value is 32-bit signed integer.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s32s32s_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s32s32s_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s32s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s32s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s32s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s32s32s_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s32s32s_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp32s* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32s* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32s*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s32s32s_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16s16sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116s16sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16s16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16s16sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16s16sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer and 16-bit signed short integer complex dot product method, return value is 16-bit signed short integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16s16sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116s16sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16s16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16s16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16s16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16s16sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16s16sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp16sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp16sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16s16sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_16sc32sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_116sc32sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_16sc32sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_16sc32sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer complex dot product method, return value is 32-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_16sc32sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_116sc32sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_16sc32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_16sc32sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    Npp16sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_16sc32sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsDotProd_32s32sc_Sfs.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProdGetBufferSize_132s32sc_1SfsNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsDotProdGetBufferSize_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProdGetBufferSize_32s32sc_Sfs(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsDotProdGetBufferSize_32s32sc_Sfs(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit signed short integer and 32-bit signed short integer complex dot product method, return value is 32-bit signed integer complex.
 * @param pSrc1 \ref source_signal_pointer.
 * @param pSrc2 \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pDp Pointer to the dot product result.
 * @param nScaleFactor \ref integer_result_scaling.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsDotProdGetBufferSize_32s32sc_Sfs to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsDotProd_132s32sc_1SfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jobject pSrc2, jint nLength, jobject pDp, jint nScaleFactor, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppsDotProd_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppsDotProd_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDp == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDp' is null for nppsDotProd_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsDotProd_32s32sc_Sfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsDotProd_32s32sc_Sfs(pSrc1=%p, pSrc2=%p, nLength=%d, pDp=%p, nScaleFactor=%d, pDeviceBuffer=%p)\n",
        pSrc1, pSrc2, nLength, pDp, nScaleFactor, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    Npp32sc* pSrc2_native = NULL;
    int nLength_native = 0;
    Npp32sc* pDp_native = NULL;
    int nScaleFactor_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDp_pointerData = initPointerData(env, pDp);
    if (pDp_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDp_native = (Npp32sc*)pDp_pointerData->getPointer(env);
    nScaleFactor_native = (int)nScaleFactor;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsDotProd_32s32sc_Sfs(pSrc1_native, pSrc2_native, nLength_native, pDp_native, nScaleFactor_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDp_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nScaleFactor is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsCountInRange_32s.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCountInRangeGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsCountInRangeGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCountInRangeGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsCountInRangeGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Computes the number of elements whose values fall into the specified range on a 32-bit signed integer array.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pCounts Pointer to the number of elements.
 * @param nLowerBound Lower bound of the specified range.
 * @param nUpperBound Upper bound of the specified range.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsCountInRangeGetBufferSize_32s to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsCountInRange_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jintArray pCounts, jint nLowerBound, jint nUpperBound, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsCountInRange_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pCounts == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pCounts' is null for nppsCountInRange_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsCountInRange_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsCountInRange_32s(pSrc=%p, nLength=%d, pCounts=%p, nLowerBound=%d, nUpperBound=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pCounts, nLowerBound, nUpperBound, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nLength_native = 0;
    int* pCounts_native;
    Npp32s nLowerBound_native;
    Npp32s nUpperBound_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pCounts_pointerData = initPointerData(env, pCounts);
    if (pCounts_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pCounts_native = (int*)pCounts_pointerData->getPointer(env);
    nLowerBound_native = (Npp32s)nLowerBound;
    nUpperBound_native = (Npp32s)nUpperBound;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsCountInRange_32s(pSrc_native, nLength_native, pCounts_native, nLowerBound_native, nUpperBound_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pCounts_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLowerBound is primitive
    // nUpperBound is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsZeroCrossing_16s32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZeroCrossingGetBufferSize_116s32fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsZeroCrossingGetBufferSize_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZeroCrossingGetBufferSize_16s32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsZeroCrossingGetBufferSize_16s32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed short integer zero crossing method, return value is 32-bit floating point.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pValZC Pointer to the output result.
 * @param tZCType Type of the zero crossing measure: nppZCR, nppZCXor or nppZCC.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsZeroCrossingGetBufferSize_16s32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZeroCrossing_116s32fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pValZC, jint tZCType, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsZeroCrossing_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pValZC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValZC' is null for nppsZeroCrossing_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // tZCType is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsZeroCrossing_16s32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZeroCrossing_16s32f(pSrc=%p, nLength=%d, pValZC=%p, tZCType=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pValZC, tZCType, pDeviceBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pValZC_native = NULL;
    NppsZCType tZCType_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pValZC_pointerData = initPointerData(env, pValZC);
    if (pValZC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValZC_native = (Npp32f*)pValZC_pointerData->getPointer(env);
    tZCType_native = (NppsZCType)tZCType;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsZeroCrossing_16s32f(pSrc_native, nLength_native, pValZC_native, tZCType_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pValZC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // tZCType is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Device-buffer size (in bytes) for nppsZeroCrossing_32f.
 * @param nLength \ref length_specification.
 * @param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZeroCrossingGetBufferSize_132fNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsZeroCrossingGetBufferSize_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZeroCrossingGetBufferSize_32f(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsZeroCrossingGetBufferSize_32f(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating-point zero crossing method, return value is 32-bit floating point.
 * @param pSrc \ref source_signal_pointer.
 * @param nLength \ref length_specification.
 * @param pValZC Pointer to the output result.
 * @param tZCType Type of the zero crossing measure: nppZCR, nppZCXor or nppZCC.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use \ref nppsZeroCrossingGetBufferSize_32f to determine the minium number of bytes required.
 * @return
 *  \ref signal_data_error_codes, \ref length_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsZeroCrossing_132fNative(JNIEnv *env, jclass cls, jobject pSrc, jint nLength, jobject pValZC, jint tZCType, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsZeroCrossing_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pValZC == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValZC' is null for nppsZeroCrossing_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // tZCType is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsZeroCrossing_32f");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsZeroCrossing_32f(pSrc=%p, nLength=%d, pValZC=%p, tZCType=%d, pDeviceBuffer=%p)\n",
        pSrc, nLength, pValZC, tZCType, pDeviceBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nLength_native = 0;
    Npp32f* pValZC_native = NULL;
    NppsZCType tZCType_native;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pValZC_pointerData = initPointerData(env, pValZC);
    if (pValZC_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValZC_native = (Npp32f*)pValZC_pointerData->getPointer(env);
    tZCType_native = (NppsZCType)tZCType;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsZeroCrossing_32f(pSrc_native, nLength_native, pValZC_native, tZCType_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pValZC_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // tZCType is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsIntegralGetBufferSize_132sNative(JNIEnv *env, jclass cls, jint nLength, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    // nLength is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppsIntegralGetBufferSize_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsIntegralGetBufferSize_32s(nLength=%d, hpBufferSize=%p)\n",
        nLength, hpBufferSize);

    // Native variable declarations
    int nLength_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    nLength_native = (int)nLength;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppsIntegralGetBufferSize_32s(nLength_native, &hpBufferSize_native);

    // Write back native variable values
    // nLength is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNpps_nppsIntegral_132sNative(JNIEnv *env, jclass cls, jobject pSrc, jobject pDst, jint nLength, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppsIntegral_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppsIntegral_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLength is primitive
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppsIntegral_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppsIntegral_32s(pSrc=%p, pDst=%p, nLength=%d, pDeviceBuffer=%p)\n",
        pSrc, pDst, nLength, pDeviceBuffer);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    Npp32s* pDst_native = NULL;
    int nLength_native = 0;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nLength_native = (int)nLength;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppsIntegral_32s(pSrc_native, pDst_native, nLength_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLength is primitive
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}


